package party.agents;

import java.util.Date;
import java.util.Random;

import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.CyclicBehaviour;
import jade.core.behaviours.OneShotBehaviour;
import jade.core.behaviours.SimpleBehaviour;
import jade.core.behaviours.WakerBehaviour;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;

@SuppressWarnings("serial")
public class GuestAgent extends Agent {

	private boolean party_mode = false;
	private boolean allInside = false;
	private AID partyHost;

	// Variables para controlar los atributos relacionados con la consumición
	private boolean eating, drinking = false;
	private int food_objective = 2;
	private int food_consumed = 0;
	private int drink_objective = 2;
	private int drink_consumed = 0;

	// Variables para controlar las paginas amarillas
	private DFAgentDescription template;
	private ServiceDescription sd;
	private DFAgentDescription template2;
	private ServiceDescription sd2;

	protected void setup() {

		registerPartyGuest(getLocalName());

		Random rnd = new Random();
		int wakeTime = (int) (rnd.nextDouble() * 10000 + 1);
		food_objective = (int) (rnd.nextDouble() * 5 + 1);
		drink_objective = (int) (rnd.nextDouble() * 5 + 1);

		template = new DFAgentDescription();
		sd = new ServiceDescription();
		sd.setType("party-host");
		template.addServices(sd);

		template2 = new DFAgentDescription();
		sd2 = new ServiceDescription();
		sd2.setType("party-guest");
		template2.addServices(sd2);

		blockingReceive(wakeTime);
		addBehaviour(new WakerBehaviour(this, wakeTime) {
			@Override
			protected void onWake() {

				party_mode = true;
				System.out.println(myAgent.getLocalName() + " [PartyGuest] [WakerBehaviour] : He llegado a la fiesta");
				myAgent.addBehaviour(new SayHelloBehaviour());
				myAgent.addBehaviour(new WelcomeBehaviour());

			}

		});

		addBehaviour(new AllInPartyListenBehaviour());

	}

	@Override
	protected void takeDown() {
		System.out.println(getLocalName() + " [PartyGuest] [takeDown] : se marcho de la fiesta y se ha comido: "
				+ food_objective + " y ha bebido: " + drink_consumed);
		removePartyGuest();
	}

	/**
	 * Metodo para registrarse como invitado
	 * 
	 * @param name
	 *            Nombre del asistente
	 */
	private void registerPartyGuest(String name) {

		/*
		 * Creamos el servicio party-guest y lo agregamos a las paginas
		 * amarillas
		 */
		DFAgentDescription dfd = new DFAgentDescription();
		dfd.setName(getAID());
		ServiceDescription sd = new ServiceDescription();
		sd.setType("party-guest");
		sd.setName(name);
		dfd.addServices(sd);

		// Registramos
		try {
			DFService.register(this, dfd);
		} catch (FIPAException fe) {
			fe.printStackTrace();
		}

	}

	/**
	 * Metodo para eliminar el registro de las paginas amarillas
	 */
	private void removePartyGuest() {
		// Eliminamos el registro de las paginas amarillas
		try {
			DFService.deregister(this);
		} catch (FIPAException fe) {
			fe.printStackTrace();
		}
	}

	private class SayHelloBehaviour extends OneShotBehaviour {

		@Override
		public void action() {
			System.out.println(myAgent.getLocalName() + " [PartyGuest] [WakerBehaviour] : Hola a tod@s");

			try {

				ACLMessage msg = new ACLMessage(ACLMessage.INFORM);
				msg.setContent("hello");
				msg.setSender(myAgent.getAID());
				partyHost = DFService.search(myAgent, template)[0].getName();
				msg.addReceiver(partyHost);

				for (DFAgentDescription df : DFService.search(myAgent, template2)) {
					msg.addReceiver(df.getName());
				}

				myAgent.send(msg);
			} catch (FIPAException fe) {
				fe.printStackTrace();
			}

		}

	}

	private class BeServedBehaviour extends CyclicBehaviour {

		@Override
		public void action() {

			ACLMessage msg = myAgent.receive(MessageTemplate.MatchPerformative(ACLMessage.PROPOSE));

			if (msg != null) {

				String text = msg.getContent();

				if (text.equals("food") || text.equals("drink")) {
					ACLMessage reply = msg.createReply();
					// reply.setPerformative(ACLMessage.ACCEPT_PROPOSAL);

					boolean wants_food = !eating && ((food_objective - food_consumed) > 0);
					boolean wants_drink = !drinking && ((drink_objective - drink_consumed) > 0);

					if ((text.equals("food") && wants_food) || (text.equals("drink") && wants_drink)) {
						reply.setPerformative(ACLMessage.ACCEPT_PROPOSAL);
						System.out.println(getLocalName() + " [PartyGuest] [BeServedBehaviour] : Si gracias señor "
								+ msg.getSender().getLocalName());
					} else {
						reply.setPerformative(ACLMessage.REJECT_PROPOSAL);
						System.out.println(myAgent.getLocalName() + " [PartyGuest] [BeServedBehaviour] : No gracias "
								+ msg.getSender().getLocalName());
					}

					myAgent.send(reply);

					// if (reply.getContent().equals("true")) {
					if (reply.getPerformative() == ACLMessage.ACCEPT_PROPOSAL) {

						MessageTemplate mt = MessageTemplate.and(
								MessageTemplate.MatchConversationId(msg.getConversationId()),
								MessageTemplate.MatchPerformative(ACLMessage.CONFIRM));

						ACLMessage msg2 = myAgent.blockingReceive(mt);
						boolean food = false;

						if (msg2 != null) {

							food = text.equals("food");

							if (food)
								eating = true;
							if (!food)
								drinking = true;

							myAgent.addBehaviour(new WakerBehaviour(myAgent, 5000) {

								@Override
								protected void onWake() {
									if (eating) {
										food_consumed++;
										System.out.println(myAgent.getLocalName()
												+ " [PartyGuest] [WakerBehaviour] : he terminado de comer este pintxito.");
										if ((food_objective == food_consumed))
											System.out.println(myAgent.getLocalName()
													+ " [PartyGuest] [WakerBehaviour] : Ya no quiero más comida");
										eating = false;
									} else {
										drink_consumed++;
										System.out.println(myAgent.getLocalName()
												+ " [PartyGuest] [WakerBehaviour] : he terminado de beber");
										if ((drink_objective == drink_consumed))
											System.out.println(myAgent.getLocalName()
													+ " [PartyGuest] [WakerBehaviour] : Ya no quiero más bebida");
										drinking = false;
									}

								}

							});
						}
					}

				}

			}

		}

	}

	private class WelcomeBehaviour extends SimpleBehaviour {

		@Override
		public void action() {

			MessageTemplate mt = MessageTemplate.and(MessageTemplate.MatchPerformative(ACLMessage.INFORM),
					MessageTemplate.MatchContent("hello"));
			ACLMessage msg = myAgent.receive(mt);
			if (msg != null && !msg.getSender().equals(myAgent.getAID())) {
				ACLMessage reply = msg.createReply();
				System.out.println(myAgent.getLocalName() + " [PartyGuest] [WelcomeBehaviour] : Hola "
						+ msg.getSender().getLocalName());
				reply.setContent("hello-back");
				myAgent.send(reply);
			} else {
				block();
			}

		}

		@Override
		public boolean done() {
			return allInside;
		}

	}

	// comportamiento para escuchar cuando el anfitrion dice que estamos todos
	private class AllInPartyListenBehaviour extends SimpleBehaviour {

		@Override
		public void action() {
			MessageTemplate mt = MessageTemplate.and(MessageTemplate.MatchPerformative(ACLMessage.INFORM),
					MessageTemplate.and(MessageTemplate.MatchContent("allInside"),
							MessageTemplate.MatchSender(partyHost)));
			ACLMessage msg = myAgent.receive(mt);
			if (msg != null) {
				allInside = true;
				System.out.println(myAgent.getLocalName()
						+ " [PartyGuest] [allInPartyListenBehaviour]: Perfecto querido anfitrión ya se que estamos todos ");
				myAgent.addBehaviour(new LeavePartyBehaviour());
				myAgent.addBehaviour(new BeServedBehaviour());
			} else {
				block();
			}

		}

		@Override
		public boolean done() {
			return allInside;
		}

	}

	private class LeavePartyBehaviour extends CyclicBehaviour {

		@Override
		public void action() {

			if (food_consumed == food_objective && drink_consumed == drink_objective && allInside) {

				System.out.println(myAgent.getLocalName()
						+ " [PartyGuest] [leavePartyBehaviour]:Ha sido una velada increible pero me tengo que marchar");

				ACLMessage msg = new ACLMessage(ACLMessage.INFORM);
				msg.setSender(myAgent.getAID());
				msg.setContent("goodbye");
				msg.addReceiver(partyHost);
				send(msg);

				MessageTemplate mt = MessageTemplate.and(MessageTemplate.MatchPerformative(ACLMessage.INFORM),
						MessageTemplate.and(MessageTemplate.MatchContent("goodbye"),
								MessageTemplate.MatchSender(partyHost)));

				ACLMessage msg2 = myAgent.blockingReceive(mt);
				if (msg2 != null) {
					myAgent.doDelete();
				}
			}
		}

	}
}
