package party.agents;

import java.io.File;

import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;

import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.Behaviour;
import jade.core.behaviours.CyclicBehaviour;
import jade.core.behaviours.OneShotBehaviour;
import jade.core.behaviours.SimpleBehaviour;
import jade.core.behaviours.TickerBehaviour;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;

public class MusicianAgent extends Agent {

	private Clip cancion;
	private AID partyHost = null;

	protected void setup() {
		registerMusician(getLocalName());
		addBehaviour(new FinishMusicBehaviour());
		addBehaviour(new SimpleBehaviour() {

			@Override
			public boolean done() {
				return partyHost != null;
			}

			@Override
			public void action() {
				try {

					DFAgentDescription template = new DFAgentDescription();
					ServiceDescription sd = new ServiceDescription();
					sd.setType("party-host");
					template.addServices(sd);

					DFAgentDescription auxHost = DFService.search(myAgent, template)[0];
					if (auxHost != null) {
						partyHost = auxHost.getName();
						addBehaviour(new allInPartyListenBehaviour());
					}
				} catch (FIPAException e) {
					e.printStackTrace();
				}
			}
		});

	}

	private void registerMusician(String name) {

		/*
		 * Creamos el servicio party-host y lo agregamos a las paginas amarillas
		 */
		DFAgentDescription dfd = new DFAgentDescription();
		dfd.setName(getAID());
		ServiceDescription sd = new ServiceDescription();
		sd.setType("party-worker");
		sd.setName(name);
		dfd.addServices(sd);

		// Registramos
		try {
			DFService.register(this, dfd);
		} catch (FIPAException fe) {
			fe.printStackTrace();
		}

	}

	/**
	 * Metodo para eliminar el registro de las paginas amarillas
	 */
	private void removePartyGuest() {
		// Eliminamos el registro de las paginas amarillas
		try {
			DFService.deregister(this);
		} catch (FIPAException fe) {
			fe.printStackTrace();
		}
	}

	private void inicializeBand() {
		try {
			Thread.sleep(1000);
			System.out.println(getLocalName() + " : 1...");
			Thread.sleep(1000);
			System.out.println(getLocalName() + " : 2...");
			Thread.sleep(1000);
			System.out.println(getLocalName() + " : 3...");
		} catch (InterruptedException e) {
			e.printStackTrace();
			System.out.println("No he podido parar el hilo");
		}
	}

	// comportamiento para escuchar cuando el anfitrion dice que estamos todos
	private class allInPartyListenBehaviour extends OneShotBehaviour {

		@Override
		public void action() {

			MessageTemplate mt = MessageTemplate.and(MessageTemplate.MatchPerformative(ACLMessage.INFORM),
					MessageTemplate.and(MessageTemplate.MatchContent("allInside"),
							MessageTemplate.MatchSender(partyHost)));
			ACLMessage msg = myAgent.blockingReceive(mt);
			if (msg != null) {

				System.out.println(
						myAgent.getLocalName() + "[allInPartyListenBehaviour]: Perfecto, ya puedo comenzar a trabajar");
				try {

					Thread.sleep(1000);
					System.out.println(getLocalName() + " : 1...");
					Thread.sleep(1000);
					System.out.println(getLocalName() + " : y 2...");
					Thread.sleep(1000);
					System.out.println(getLocalName() + " : y 3...");

					myAgent.addBehaviour(new PlayMusic(myAgent, 7450));
					myAgent.addBehaviour(new FinishMusicBehaviour());

					AudioInputStream audioInputStream = AudioSystem.getAudioInputStream(new File(
							"src/party/agents/music.wav"/*
														 * nombre de la cancion
														 */).getAbsoluteFile());
					cancion = AudioSystem.getClip();
					cancion.open(audioInputStream);
					cancion.start();
				} catch (Exception e) {
					System.out.println(e);
					System.out.println("No se ha cargado la musica");
				}
			}

		}

		private class PlayMusic extends TickerBehaviour {

			public PlayMusic(Agent a, long period) {
				super(a, period);
			}

			@Override
			protected void onTick() {
				try {
					AudioInputStream audioInputStream = AudioSystem
							.getAudioInputStream(new File("src/party/agents/music.wav").getAbsoluteFile());
					cancion = AudioSystem.getClip();
					cancion.open(audioInputStream);
					cancion.start();
				} catch (Exception e) {
					System.out.println(e);
					System.out.println("No se ha cargado la musica");
				}

			}

		}

	}

	private class FinishMusicBehaviour extends CyclicBehaviour {

		@Override
		public void action() {
			MessageTemplate mt = MessageTemplate.and(MessageTemplate.MatchSender(partyHost), MessageTemplate.and(
					MessageTemplate.MatchPerformative(ACLMessage.REQUEST), MessageTemplate.MatchContent("finish")));
			ACLMessage msg = myAgent.receive(mt);
			if (msg != null) {
				ACLMessage reply = msg.createReply();
				reply.setConversationId(msg.getConversationId());
				reply.setPerformative(ACLMessage.AGREE);
				System.out.println(myAgent.getLocalName() + " [FinishMusicBehaviour] Ya es hora de marcharse");
				myAgent.send(reply);
				myAgent.doDelete();
			} else {
				block();
			}
		}

	}
}
