package party.agents;

import java.util.ArrayList;

import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.CyclicBehaviour;
import jade.core.behaviours.SimpleBehaviour;
import jade.core.behaviours.TickerBehaviour;
import jade.core.behaviours.WakerBehaviour;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;

@SuppressWarnings("serial")
public class PartyHostAgent extends Agent {

	private ArrayList<AID> guests;
	private boolean allInParty = false;

	private DFAgentDescription template;
	private ServiceDescription sd;
	private DFAgentDescription template2;
	private ServiceDescription sd2;

	private boolean eating, drinking = false;
	private int food_consumed = 0;
	private int drink_consumed = 0;

	protected void setup() {

		// Cargando los templates al inicio
		template = new DFAgentDescription();
		sd = new ServiceDescription();
		sd.setType("party-guest");
		template.addServices(sd);
		template2 = new DFAgentDescription();
		sd2 = new ServiceDescription();
		sd2.setType("party-worker");
		template2.addServices(sd2);

		System.out.println(getLocalName() + " [PartyHost] : Bienvenidos a mi fiesta, ya se puede pasar");

		guests = new ArrayList<>();

		registerPartyHost(getLocalName());
		addBehaviour(new WelcomeBehaviour());
		addBehaviour(new TickerBehaviour(this, 5000) {
			/**
			 * comportamiento para ver cada 5 segundos han llegado todos
			 * 
			 */
			@Override
			protected void onTick() {

				try {
					if (DFService.search(myAgent, template).length == guests.size()) {

						System.out.println(myAgent.getLocalName()
								+ " [PartyHost] [TickerBehaviour] : Ya estamos todos los importantes en la fiesta, Que comience la marcha");
						allInParty = true;
						try {
							ACLMessage msg = new ACLMessage(ACLMessage.INFORM);
							msg.setContent("allInside");
							msg.setSender(myAgent.getAID());

							for (DFAgentDescription df : DFService.search(myAgent, template)) {
								msg.addReceiver(df.getName());
							}

							for (DFAgentDescription df : DFService.search(myAgent, template2)) {
								msg.addReceiver(df.getName());
							}

							myAgent.send(msg);
							myAgent.addBehaviour(new BeServedBehaviour());
							myAgent.addBehaviour(new SayGoodByeBehaviour());
							myAgent.addBehaviour(new ControlPartyStateBehaviour());

						} catch (FIPAException fe) {
							fe.printStackTrace();
						}
						myAgent.removeBehaviour(this);
					} else {
						System.out.println(myAgent.getLocalName()
								+ " [PartyHost] [TickerBehaviour] : Parece que aún no estamos todos.");
					}
				} catch (FIPAException e) {
					e.printStackTrace();
				}

			}
		});

	}

	@Override
	protected void takeDown() {
		System.out.println(getLocalName() + " [PartyHost] [takeDown] : La fiesta ha sido sublime, la próxima más");
		System.out.println(getLocalName() + " [PartyHost] [takeDown] : pero no mejor, porque es imposible");
		System.out.println(getLocalName() + " [PartyHost] [takeDown] : Total de bebidas = " + drink_consumed
				+ " Total de comidas = " + food_consumed);
		removePartyHostService();
	}

	/**
	 * Metodo para registrarse como party host
	 * 
	 * @param name
	 *            Nombre de la fiesta
	 */
	private void registerPartyHost(String name) {

		/*
		 * Creamos el servicio party-host y lo agregamos a las paginas amarillas
		 */
		DFAgentDescription dfd = new DFAgentDescription();
		dfd.setName(getAID());
		ServiceDescription sd = new ServiceDescription();
		sd.setType("party-host");
		sd.setName(name);
		dfd.addServices(sd);

		// Registramos
		try {
			DFService.register(this, dfd);
		} catch (FIPAException fe) {
			fe.printStackTrace();
		}

	}

	/**
	 * Metodo para eliminar el registro de las paginas amarillas
	 */
	private void removePartyHostService() {
		// Eliminamos el registro de las paginas amarillas
		try {
			DFService.deregister(this);
		} catch (FIPAException fe) {
			fe.printStackTrace();
		}
	}

	private class WelcomeBehaviour extends SimpleBehaviour {

		@Override
		public void action() {

			MessageTemplate mt1 = MessageTemplate.MatchPerformative(ACLMessage.INFORM);
			MessageTemplate mt2 = MessageTemplate.MatchContent("hello");

			MessageTemplate mt = MessageTemplate.and(MessageTemplate.MatchPerformative(ACLMessage.INFORM),
					MessageTemplate.MatchContent("hello"));

			ACLMessage msg = myAgent.receive(mt);
			if (msg != null) {

				try {

					for (DFAgentDescription da : DFService.search(myAgent, template)) {

						if (da.getName().equals(msg.getSender())) {

							ACLMessage reply = msg.createReply();
							System.out.println(myAgent.getLocalName()
									+ " [PartyHost] [WelcomeBehaviour] : Le devuelvo el saludo querid@ invitad@ "
									+ msg.getSender().getLocalName());

							reply.setContent("hello-back");

							if (!guests.contains(msg.getSender())) {
								guests.add(msg.getSender());
							}

							myAgent.send(reply);
							break;
						}
					}
				} catch (FIPAException e) {
					e.printStackTrace();
				}

			}

		}

		@Override
		public boolean done() {
			return allInParty;
		}

	}

	private class ControlPartyStateBehaviour extends CyclicBehaviour {

		@Override
		public void action() {
			try {
				if (DFService.search(myAgent, template).length == 0
						&& DFService.search(myAgent, template2).length == 1) {
					AID musician = DFService.search(myAgent, template2)[0].getName();
					String conversationId = "finish:" + getLocalName() + "-" + musician.getLocalName();

					System.out.println(myAgent.getLocalName()
							+ " [PartyHost] [ControlPartyStateBehaviour] : ('plof plof') Fin de la fiesta, fuera música.");

					ACLMessage msg = new ACLMessage(ACLMessage.REQUEST);
					msg.setContent("finish");
					msg.addReceiver(musician);
					msg.setConversationId(conversationId);
					myAgent.send(msg);

					MessageTemplate mt = MessageTemplate.and(MessageTemplate.MatchConversationId(conversationId),
							MessageTemplate.MatchPerformative(ACLMessage.AGREE));
					ACLMessage msg2 = myAgent.blockingReceive(mt);
					if (msg2 != null) {
						myAgent.doDelete();
					}
				}

			} catch (FIPAException e) {
				e.printStackTrace();
			}
		}

	}

	private class SayGoodByeBehaviour extends SimpleBehaviour {

		@Override
		public void action() {
			MessageTemplate mt = MessageTemplate.and(MessageTemplate.MatchPerformative(ACLMessage.INFORM),
					MessageTemplate.MatchContent("goodbye"));

			ACLMessage msg = myAgent.receive(mt);
			if (msg != null && guests.contains(msg.getSender())) {
				ACLMessage reply = msg.createReply();
				System.out.println(myAgent.getLocalName() + " [PartyHost] [SayGoodBye] : La siguiente en tu casa "
						+ msg.getSender().getLocalName());
				reply.setContent("goodbye");
				myAgent.send(reply);
				guests.remove(msg.getSender());
			} else {
				block();
			}
		}

		@Override
		public boolean done() {
			// TODO Auto-generated method stub
			return guests.size() == 0;
		}

	}

	private class BeServedBehaviour extends CyclicBehaviour {

		@Override
		public void action() {

			ACLMessage msg = myAgent.receive(MessageTemplate.MatchPerformative(ACLMessage.PROPOSE));

			if (msg != null) {

				String text = msg.getContent();

				if (text.equals("food") || text.equals("drink")) {
					ACLMessage reply = msg.createReply();
					// reply.setPerformative(ACLMessage.ACCEPT_PROPOSAL);

					boolean wants_food = !eating;
					boolean wants_drink = !drinking;

					if ((text.equals("food") && wants_food) || (text.equals("drink") && wants_drink)) {
						reply.setPerformative(ACLMessage.ACCEPT_PROPOSAL);
						System.out.println(getLocalName() + " [PartyHost] [BeServedBehaviour] : Sí, gracias señor "
								+ msg.getSender().getLocalName());
					} else {
						reply.setPerformative(ACLMessage.REJECT_PROPOSAL);
						System.out.println(myAgent.getLocalName() + " [PartyHost] [BeServedBehaviour] : No, gracias "
								+ msg.getSender().getLocalName());
					}

					myAgent.send(reply);

					if (reply.getPerformative() == ACLMessage.ACCEPT_PROPOSAL) {

						MessageTemplate mt = MessageTemplate.and(
								MessageTemplate.MatchConversationId(msg.getConversationId()),
								MessageTemplate.MatchPerformative(ACLMessage.CONFIRM));

						ACLMessage msg2 = myAgent.blockingReceive(mt);
						boolean food = false;

						if (msg2 != null) {

							food = text.equals("food");

							if (food)
								eating = true;
							if (!food)
								drinking = true;

							myAgent.addBehaviour(new WakerBehaviour(myAgent, 5000) {

								@Override
								protected void onWake() {
									String texto = myAgent.getLocalName() + " [PartyHost] [WakerBehaviour] : ";
									if (eating) {
										food_consumed++;
										System.out.println(myAgent.getLocalName()
												+ " [PartyHost] [WakerBehaviour] : he terminado de comer este pintxito.");

										switch (food_consumed) {
										case 1:
											texto = texto + "Solo ofrezco la comida mejor calidad.";
											break;
										case 2:
											texto = texto + "Creo que ya me estoy pasando.";
											break;
										default:
											texto = texto + "Como coma más ¡EXPLOTO!.";
											break;
										}

										eating = false;
									} else {
										drink_consumed++;
										System.out.println(myAgent.getLocalName()
												+ " [PartyGuest] [WakerBehaviour] : he terminado de beber");

										switch (drink_consumed) {
										case 1:
											texto = texto + "Si que está flojo el coctel sí.";
											break;
										case 2:
											texto = texto + "Creo que estoy pillando el punto.";
											break;
										default:
											texto = texto + "¡¡FIESTA!! ¡¡¡¡SUBIDÓN!!!!.";
											break;
										}

										drinking = false;
									}
									System.out.println(texto);

								}

							});
						}
					}

				}

			}

		}

	}

}
