package party.agents;

import java.util.Random;

import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.SimpleBehaviour;
import jade.core.behaviours.TickerBehaviour;
import jade.core.behaviours.WakerBehaviour;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;

@SuppressWarnings("serial")
public class WaiterAgent extends Agent {

	private boolean serves_food;
	private String item = "bebida";
	private AID partyHost = null;
	private boolean allInside = false;
	private int worked_time;


	protected void setup() {

		serves_food = getArguments()[0].equals("true");
		if (serves_food)
			item = "comida";

		registerPartyWaiter(getLocalName());

		DFAgentDescription template = new DFAgentDescription();
		ServiceDescription sd = new ServiceDescription();
		sd.setType("party-host");
		template.addServices(sd);

		addBehaviour(new SimpleBehaviour() {

			@Override
			public boolean done() {
				return partyHost != null;
			}

			@Override
			public void action() {
				try {

					DFAgentDescription auxHost = DFService.search(myAgent, template)[0];
					if (auxHost != null) {
						partyHost = auxHost.getName();
						addBehaviour(new allInPartyListenBehaviour());
					}
				} catch (FIPAException e) {
					e.printStackTrace();
				}
			}
		});
	}

	@Override
	protected void takeDown() {
		System.out.println(getLocalName() + " [Waiter] [takeDown] : he terminado mi trabajo y me marcho de la fiesta");
		System.out.println(getLocalName() + " [Waiter] [takeDown] : he servido un total de "+worked_time+" veces.");
		removePartyWaiter();
	}

	private void registerPartyWaiter(String name) {

		/*
		 * Creamos el servicio party-host y lo agregamos a las paginas amarillas
		 */
		DFAgentDescription dfd = new DFAgentDescription();
		dfd.setName(getAID());
		ServiceDescription sd = new ServiceDescription();
		sd.setType("party-worker");
		sd.setName(name);
		dfd.addServices(sd);

		// Registramos
		try {
			DFService.register(this, dfd);
		} catch (FIPAException fe) {
			fe.printStackTrace();
		}

	}

	/**
	 * Metodo para eliminar el registro de las paginas amarillas
	 */
	private void removePartyWaiter() {
		// Eliminamos el registro de las paginas amarillas
		try {
			DFService.deregister(this);
		} catch (FIPAException fe) {
			fe.printStackTrace();
		}
	}

	// comportamiento para escuchar cuando el anfitrion dice que estamos todos
	private class allInPartyListenBehaviour extends SimpleBehaviour {

		@Override
		public void action() {

			MessageTemplate mt = MessageTemplate.and(MessageTemplate.MatchPerformative(ACLMessage.INFORM),
					MessageTemplate.and(MessageTemplate.MatchContent("allInside"),
							MessageTemplate.MatchSender(partyHost)));
			ACLMessage msg = myAgent.receive(mt);
			if (msg != null) {
				allInside = true;
				System.out.println(myAgent.getLocalName()
						+ " [Waiter] [allInPartyListenBehaviour]: Perfecto, ya puedo comenzar a trabajar");
				

				myAgent.addBehaviour(new ServeItemBehaviour(myAgent,5000));

			}

		}

		@Override
		public boolean done() {
			return allInside;
		}

	}

	private class ServeItemBehaviour extends TickerBehaviour {

		public ServeItemBehaviour(Agent a, long period) {
			super(a, period);
			// TODO Auto-generated constructor stub
		}

		@Override
		protected void onTick() {
			worked_time++;
			DFAgentDescription template = new DFAgentDescription();
			ServiceDescription sd = new ServiceDescription();
			sd.setType("party-guest");
			template.addServices(sd);

			try {

				DFAgentDescription invitados[] = DFService.search(myAgent, template);
				if (invitados.length == 0) {
					myAgent.doDelete();
				} else {
					Random rnd = new Random();
					int selected = (int) (rnd.nextDouble() * (invitados.length + 1));
					ACLMessage msg = new ACLMessage(ACLMessage.PROPOSE);
					msg.setSender(myAgent.getAID());

					AID receiver = null;

					if (selected == invitados.length)
						receiver = partyHost;
					else
						receiver = invitados[selected].getName();

				
					msg.addReceiver(receiver);

					String content = "food";

					if (!serves_food)
						content = "drink";

					String conversationid = content + "/-/" + receiver.getName();
					msg.setContent(content);

					msg.setConversationId(conversationid);

					System.out.println(getLocalName() + " [Waiter] [ServeItemBehaviour] : ¿ Desea " + item + " "
							+ receiver.getLocalName() + " ?");
					myAgent.send(msg);

					
					MessageTemplate mt = MessageTemplate.MatchConversationId(conversationid);

					ACLMessage msg2 = myAgent.blockingReceive(mt,3000);

					if (msg2 != null && msg2.getPerformative() == ACLMessage.ACCEPT_PROPOSAL) {
						ACLMessage reply = msg2.createReply();
						reply.setPerformative(ACLMessage.CONFIRM);
						System.out.println(getLocalName() + " [Waiter] [ServeItemBehaviour] : Aquí tienes "
								+ msg2.getSender().getLocalName());
						myAgent.send(reply);
					}

				}

			} catch (FIPAException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}

	}
}
